/*********************************************************************
*  File name: PWM3 TMR2　音声再生
*  Notes: 32MHz内部クロック　PLLはOn
*        PWM    RC5
*        PushSW RC4
* PIC16F1619  MPLAB X IDE with XC8
* 2020-4-4
* Copyright (c) 2020 iwamoto All Rights Reserved
* *******************************************************************/
#include <xc.h>
#include "wOha.h" // include　PCNデータ

#define _XTAL_FREQ 32000000

// CONFIG
#pragma config FOSC = INTOSC, PWRTE = OFF, MCLRE = ON, CP = OFF
#pragma config BOREN = ON, CLKOUTEN = OFF, IESO = ON, FCMEN = ON
#pragma config WRT = OFF, PPS1WAY = ON, ZCD = OFF, PLLEN = ON
#pragma config STVREN = ON, BORV = LO, LPBOR = OFF, LVP = ON
#pragma config WDTCPS = WDTCPS1F, WDTE = OFF, WDTCWS = WDTCWSSW, WDTCCS = SWC

/*********************************************************
                         Main application
 *********************************************************/
void main(void)
{
    int count = 0;

    OSCCON = 0b11110000;	// PLL ON; 8Hz_HF; FOSC;
    TRISC5 = 0;         // Outputs RC5 for PWM3
    RC5PPS = 14;        // PWM3をRC3に設定
    // --------------------------------------------
    // PWM3は、TMR"と連携し、
    // 初期値デュティサイクルは、50%
    // --------------------------------------------
    PWM3CON = 0b10000000;         // PWM ON
    PWM3DCH = 0x00;               // デュティサイクルを設定
    CCPTMRS = 0x00;               // TMR2と連携
    // --------------------------------------------
    // Timer2 は、Fosc/4(8MHz)をPreScalerで 1/1(2MHz)
    // ８bitで255カウントする　　　　　8M / 255 = 31KHz
    // PostScaler 1/4 でTMR2IFが「１」になる 7.8KHz周期
    // TMR2IFを常時モニタし、LED(RC0)を反転
    // --------------------------------------------
    T2CON  = 0b10000011;      // Timer2 Pre1/1 Post1/4設定
    PR2    = 255;             // Timer2 Period Register設定
    T2CLKCON = 0x00;          // クロックはFosc/4
    T2HLT    = 0x00;          // Free Runモード
    T2RST    = 0x00;          // Free Runなので該当せず

    while(1){                 // 繰り返しループ
       while(! TMR2IF);       // Timer2 8KHzのカウントアップを待つ
        TMR2IF = 0;           // キャリーフラッグをクリア
        PWM3DCH = wavedata[count];    // デュティサイクルを設定
        if(wavedata[count++]==0){     // 音声データ終了なら
            count = 0;                // データポインタを戻す
            __delay_ms(1000);         // １秒待つ
            while(RC4);               // PushSWが押されるのを待つ
        }
    }
}
